@@ -10,10 +10,10 @@ from __future__ import annotations
 
 import asyncio
 import logging
+import pathlib
 import warnings
-from os import PathLike
 
-from typing import Any, Optional, TypeVar, cast
+from typing import Any, Optional, TypeVar, Union, cast
 
 import clu
 
@@ -30,6 +30,7 @@ __all__ = ["LegacyActor", "BaseLegacyActor"]
 
 
 T = TypeVar("T", bound="BaseLegacyActor")
+PathLike = Union[str, pathlib.Path]
 
 
 class BaseLegacyActor(BaseActor):
@@ -419,7 +420,10 @@ class BaseLegacyActor(BaseActor):
 
         lines = []
         for keyword in message:
-            value = clu.format_value(message[keyword])
+            try:
+                value = clu.format_value(message[keyword])
+            except BaseException as err:
+                raise TypeError(f"Cannot format keyword {keyword!r} " + str(err))
             lines.append(f"{keyword}={value}")
 
         if concatenate:
@@ -438,7 +442,8 @@ class BaseLegacyActor(BaseActor):
             else:
                 transport.write(msg)
 
-            log_reply(self.log, message_code, full_msg_str)
+            if self.log:
+                log_reply(self.log, message_code, full_msg_str)
 
 
 class LegacyActor(ClickParser, BaseLegacyActor):
